unit Histfrm;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Header, PhBook, StdCtrls, Buttons;

type
  THistory = class(TStringList)
  private
    MaxHistory: Integer;
  public
    procedure Read;
    procedure Write;
    procedure AddEntry(number, name: String);
  end;

  THistForm = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    PhBookBtn: TBitBtn;
    LBAlias: TListBox;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PhBookBtnClick(Sender: TObject);
    procedure LBAliasClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  HistForm: THistForm;
  HistoryList : THistory;

implementation

{$R *.DFM}

procedure THistory.Read;
{ Read history from ini file }
var
  i : Integer;
  p, p1 : array[0..255]of Char;
  p2 : PChar;
begin
  MaxHistory := GetPrivateProfileInt( OptionsSection, MaxHistKey, KMaxHistory, WinIni );
  if MaxHistory > KMaxHistory then MaxHistory := KMaxHistory;
  for i := Count-1 downto 0 do StrDispose( PChar(Objects[i]) );
  if GetPrivateProfileInt( OptionsSection, SaveHistoryKey, 0, WinIni ) <> 0 then begin
    for i := 0 to MaxHistory do begin  { fill history }
      StrPCopy( @p, 'History'+IntToStr(i) );
      GetPrivateProfileString( HistorySection, @p, '', @p1, 255, IniName );
      p2 := StrScan( @p1, ':' );
      if p2 <> nil then begin
        p2^ := #0;
        Inc( p2 );
        while p2^ = ' ' do Inc( p2 );
        AddObject( StrPas( @p1 ), TObject(StrNew( p2 )));
      end;
    end;
  end;
end;

procedure THistory.Write;
{ Write history to ini file }
var
  i : Integer;
  p,p1: array[0..255]of Char;
begin
  i := 0;
  if GetPrivateProfileInt( OptionsSection, SaveHistoryKey, 0, WinIni ) <> 0 then begin
    while i < Count do begin
      StrPCopy( @p, 'History'+IntToStr(i) );
      StrPCopy( @p1, Strings[i] );
      StrLCat( @p1, ': ', 255 );
      StrLCat( @p1, PChar(Objects[i]), 255 );
      WritePrivateProfileString( HistorySection, @p, @p1, IniName );
      Inc(i);
    end;
  end;
  while i < KMaxHistory do begin
    StrPCopy( @p, 'History'+IntToStr(i) );
    WritePrivateProfileString( HistorySection, @p, nil, IniName );
    Inc(i);
  end;
  WritePrivateProfileString( nil, nil, nil, IniName );
end;

procedure THistory.AddEntry(number, name: String);
{ Add an entry to history }
var
  p : array[0..255]of Char;
  pn : PChar;
  i : Integer;
begin
  StrPCopy( @p, name );
  for i := 0 to Count-1 do
    if (Strings[i] = number) and (StrComp(@p,PChar(Objects[i]))=0) then begin
      Move(i, 0);
      exit;
    end;
  with PhoneBook do for i := 0 to Count-1 do
    if (PAlias(Objects[i])^.FaxNum = number) and
       (PAlias(Objects[i])^.Empfaenger = name) then exit;
  pn := StrNew( @p );
  if not ( Count < MaxHistory ) then
    if Count > 0 then begin
      StrDispose(PChar(Objects[0]));
      Delete( 0 );
    end;
  try
    InsertObject( 0, number, TObject(pn) );
  except
    On EOutOfMemory do StrDispose(pn);
  end;
end;

procedure THistForm.FormCreate(Sender: TObject);
begin
  Caption := CProgName+' '+GetLabelStr(KIHistory);
  OKBtn.Caption := GetLabelStr(KIOK);
  CancelBtn.Caption := GetLabelStr(KICancel);
  PhBookBtn.Caption := UnShortCut(GetLabelStr(KIPhoneBook));
end;

procedure THistForm.FormShow(Sender: TObject);
{ Initialize list and phonebook button }
begin
  LBAlias.ItemIndex := 0;
  PhBookBtn.Enabled := HistoryList.Count > 0;
end;

procedure THistForm.PhBookBtnClick(Sender: TObject);
{ ShowModal phonebook form with some edit fields preset from
  selected list item }
var
  i, j : Integer;
  s : String;
begin
  j := LBAlias.ItemIndex;
  if (j >= 0) and (j < HistoryList.Count) then begin
    s := LBAlias.Items.Strings[j];
    i := Pos(':',s);
    if i > 1 then begin
      CreatePBookForm(Self);
      PhBookForm.FaxNum.Text := Copy( s, 1, i-1 );
      PhBookForm.Empfaenger.Text := Copy( s, i+2, Length( s ));
      ExecutePBookForm;
    end;
  end;
end;

procedure THistForm.LBAliasClick(Sender: TObject);
{ Replacement for OnChange list event }
var
  j : Integer;
begin
  j := LBAlias.ItemIndex;
  PhBookBtn.Enabled := ((j >= 0) and (j < HistoryList.Count));
end;

end.
