unit Phbook;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, Grids, Header;

const
  KGetMem = 2048;

type
  { Phonebook entry }
  TAlias = record
    FaxNum, Empfaenger: String;
  end;
  PAlias = ^TAlias;

  TPhoneBook = class(TStringList)
  public
    procedure Read;
    procedure Update;
  end;

  TPhBookForm = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    AliasList: TListBox;
    PlusBtn: TButton;
    MinusBtn: TButton;
    Alias: TEdit;
    FaxNum: TEdit;
    Empfaenger: TEdit;
    EmpfaengerLabel: TLabel;
    FaxNumLabel: TLabel;
    AliasLabel: TLabel;
    AliasListLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure AliasListClick(Sender: TObject);
    procedure PlusBtnClick(Sender: TObject);
    procedure MinusBtnClick(Sender: TObject);
    procedure AliasChange(Sender: TObject);
    procedure AliasListExit(Sender: TObject);
    procedure AliasListEnter(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  PhBookForm: TPhBookForm;
  PhoneBook: TPhoneBook;

procedure CreatePBookForm(AOwner: TForm);
procedure ExecutePBookForm;

implementation

{$R *.DFM}

procedure TPhoneBook.Read;
{ Read phonebook from ini file }
var
  i : Integer;
  p : array[0..255]of Char;
  p2, p3 : PChar;
  a : PAlias;
  s : String;
begin
  Getmem( p2, KGetMem );
  GetPrivateProfileString( AliasSection, nil, '', p2, KGetMem, IniName );
  p3 := p2;
  while p3^ <> #0 do begin
    GetPrivateProfileString( AliasSection, p3, '', @p, 255, IniName );
    s := StrPas( @p );
    i := Pos( ':', s );
    New(a);
    a^.FaxNum := Copy( s, 1, i-1 );
    a^.Empfaenger := Copy( s, i+2, length(s) );
    try
      AddObject( StrPas( p3 ), TObject(a) );
    except
      on EOutOfMemory do Dispose(a);
    end;
    p3 := StrEnd( p3 );
    Inc(p3);
  end;
  FreeMem( p2, KGetMem );
end;

procedure TPhoneBook.Update;
{ Write phonebook to ini file }
var
  i : Integer;
  p, p1 : array[0..255]of Char;
begin
  if PhBookForm = nil then exit;
  for i := 0 to Count-1 do begin
    if PhBookForm.AliasList.Items.IndexOf(Strings[i]) < 0 then begin
      { remove alias key from ini file if it was deleted }
      StrPCopy( @p, Strings[i] );
      WritePrivateProfileString( AliasSection, @p, nil, IniName );
    end;
    Dispose( PAlias(Objects[i]) );
  end;
  Clear;  { Clear phonebook... }
  with PhBookForm.AliasList do for i := 0 to Items.Count-1 do begin
    { ...and copy all entries from form alias list to phonebook... }
    Self.AddObject( Items.Strings[i], Items.Objects[i] );
    StrPCopy( @p, Items.Strings[i] );
    StrPCopy( @p1, PAlias(Items.Objects[i])^.FaxNum+': '+
                  PAlias(Items.Objects[i])^.Empfaenger );
    { ...and write to ini file }
    WritePrivateProfileString( AliasSection, @p, @p1, IniName );
  end;
  { Flush ini file }
  WritePrivateProfileString( nil, nil, nil, IniName );
end;

procedure TPhBookForm.FormCreate(Sender: TObject);
begin
  Caption := CProgName+' '+UnShortCut(StrPas(PBookMenuItem));
  OKBtn.Caption := GetLabelStr(KIOK);
  CancelBtn.Caption := GetLabelStr(KICancel);
  AliasLabel.Caption := GetLabelStr(KIAlias);
  AliasListLabel.Caption := GetLabelStr(KIAlias);
  FaxNumLabel.Caption := GetLabelStr(KIFaxNums);
  EmpfaengerLabel.Caption := GetLabelStr(KIRecvName);
end;

procedure TPhBookForm.AliasListClick(Sender: TObject);
{ Replacement for the OnChange list event }
var
  a: PAlias;
begin
  with AliasList do begin
    if ItemIndex >= 0 then begin
      Alias.Text := Items.Strings[ItemIndex];
      a := PAlias(Items.Objects[ItemIndex]);
      FaxNum.Text := a^.FaxNum;
      Empfaenger.Text := a^.Empfaenger;
      PlusBtn.Default := False;
      OKBtn.Default := False;
      MinusBtn.Default := True;
    end;
  end;
end;

procedure TPhBookForm.PlusBtnClick(Sender: TObject);
{ Add an alias }
var
  a: PAlias;
  i: Integer;
begin
  if ( Alias.Text <> '' ) and ( FaxNum.Text <> '' ) then begin
    New(a);
    a^.FaxNum := FaxNum.Text;
    a^.Empfaenger := Empfaenger.Text;
    with AliasList do begin
      i := Items.IndexOf( AnsiLowerCase(Alias.Text) );
      if i >= 0 then begin
        Dispose( PAlias(Items.Objects[i]) );
        Items.Delete( i );
      end;
      try
        Items.AddObject( AnsiLowerCase(Alias.Text), TObject(a) );
        Alias.Text := '';
        FaxNum.Text := '';
        Empfaenger.Text := '';
      except
        on EOutOfMemory do Dispose(a);
      end;
    end;
  end;
end;

procedure TPhBookForm.MinusBtnClick(Sender: TObject);
{ Remove an alias }
var
  i: Integer;
  a : PAlias;
begin
  with AliasList do begin
    for i := 0 to Items.Count-1 do
    if Selected[i] then begin
      Alias.Text := Items.Strings[i];
      a := PAlias(Items.Objects[i]);
      FaxNum.Text := a^.FaxNum;
      Empfaenger.Text := a^.Empfaenger;
      Dispose(a);
      Items.Delete( i );
      break;
    end;
    if Items.Count > 0 then
      SetFocus
    else
      Alias.SetFocus;
  end;
end;

procedure CreatePBookForm(AOwner: TForm);
{ Create phonebook from, set labels and fill alias list;
  don't ShowModal it now, to allow presetting of some edit fields }
var
  i : Integer;
  a : PAlias;
begin
  PhBookForm := TPhBookForm.Create(AOwner);
  for i := 0 to PhoneBook.Count-1 do begin
    New(a);
    a^ := PAlias(PhoneBook.Objects[i])^;
    try
      PhBookForm.AliasList.Items.AddObject( PhoneBook.Strings[i], TObject(a) );
    except
      on EOutOfMemory do Dispose(a);
    end;
  end;
end;

procedure ExecutePBookForm;
{ ShowModal the prepared phonebook form }
var
  i : Integer;
begin
  with PhBookForm do begin
    if ShowModal = IDOK then PhoneBook.Update
    else begin
      for i := 0 to AliasList.Items.Count-1 do
        Dispose( PAlias(AliasList.Items.Objects[i]) );
    end;
  end;
  PhBookForm.Free;
end;

procedure TPhBookForm.AliasChange(Sender: TObject);
begin
  if not AliasList.Focused then begin
    PlusBtn.Default := Alias.Text <> '';;
    OKBtn.Default := not PlusBtn.Default;
  end;
end;

procedure TPhBookForm.AliasListExit(Sender: TObject);
begin
  if MinusBtn.Default = True then begin
    MinusBtn.Default := False;
    if Alias.Text = '' then OKBtn.Default := True
                       else PlusBtn.Default := True;
  end;
end;

procedure TPhBookForm.AliasListEnter(Sender: TObject);
begin
  if AliasList.ItemIndex >= 0 then begin
    PlusBtn.Default := False;
    OKBtn.Default := False;
    MinusBtn.Default := True;
  end;
end;

end.
